@@ -1,13 +1,15 @@
 from __future__ import unicode_literals
 
+import base64
 import logging
 import time
 import traceback
 import random
-
 import requests
+from datetime import datetime
+
 from MammothAnalytics import const
-from MammothAnalytics.const import USER_PERMISSIONS,RESERVED_BATCH_COLUMN_INTERNAL_NAMES_AND_KEYS
+from MammothAnalytics.const import USER_PERMISSIONS, RESERVED_BATCH_COLUMN_INTERNAL_NAMES_AND_KEYS
 from MammothAnalytics.errors import AuthError, UnknownError
 from MammothAnalytics.errors import MalformedRequestError, AuthenticationError, \
     NotFoundError, AuthorizationError
@@ -44,8 +46,18 @@ def handleError(f):
     return new_function
 
 
+def encode_integration_key(key):
+    """encrypt the base 64 encrypted integration key to escape adblocker
+    Args:
+        key (string): base 64 encrypted integration key of the connector
+    Returns:
+        string: integration string of the connector
+    """
+    return base64.b64encode(key.encode('ascii')).decode()
+
+
 class MammothConnector(object):
-    def __init__(self, email, password=None, account_id=None, token=None, api_url=None):
+    def __init__(self, email=None, password=None, account_id=None, token=None, api_url=None, api_key=None, api_secret=None):
 
         """
         The main class for handling Mammoth Analytics API
@@ -61,16 +73,26 @@ class MammothConnector(object):
         if token:
             self.__token = token
         elif password:
-            response = requests.post(get_url('/login', self.api_url), json=dict(email=email, password=password, is_long_term=True))
+            response = requests.post(get_url('/login', self.api_url),
+                                     json=dict(email=email, password=password, is_long_term=True))
             if response.status_code != 200:
-                raise AuthError(response.json()["ERROR_CODE"],'{0}'.format(response.status_code, response.json()['ERROR_MESSAGE']))
+                raise AuthError(response.json()["ERROR_CODE"],
+                                '{0}'.format(response.status_code, response.json()['ERROR_MESSAGE']))
             self.__token = response.json()['token']
-            response = requests.post(get_url('/login', self.api_url), json=dict(email=email, password=password, is_long_term=True))
+            response = requests.post(get_url('/login', self.api_url),
+                                     json=dict(email=email, password=password, is_long_term=True))
             if response.status_code != 200:
                 raise AuthError("username or password is not correct")
             self.__token = response.json()['token']
+        elif api_key and api_secret:
+            response = requests.post(get_url('/login', self.api_url),
+                                     json=dict(api_key=api_key, api_secret=api_secret, is_long_term=True))
+            if response.status_code != 200:
+                raise AuthError(response.json()["ERROR_CODE"],
+                                '{0}'.format(response.status_code, response.json()['ERROR_MESSAGE']))
+            self.__token = response.json()['token']
         else:
-            raise AuthError("Token or Password needed!")
+            raise AuthError("Token /Password /Appkey Appsecret needed!")
 
         if account_id is not None:
             self.select_account(account_id)
@@ -107,12 +129,16 @@ class MammothConnector(object):
             resp = response.json()
             log.debug('response json :{0}'.format(resp))
         except Exception as e:
-            raise UnknownError(resp['ERROR_CODE'], 'Server responded with status code :{0}'.format(response.status_code, resp['ERROR_MESSAGE']))
+            raise UnknownError(resp['ERROR_CODE'], 'Server responded with status code :{0}'.format(response.status_code,
+                                                                                                   resp[
+                                                                                                       'ERROR_MESSAGE']))
         if response.status_code == 200:
             return response
         elif response.status_code == 500:
             if resp['ERROR_CODE'] == 0:
-                raise UnknownError(resp['ERROR_CODE'], 'Server responded with status code :{0} and message:'.format(response.status_code, resp['ERROR_MESSAGE']))
+                raise UnknownError(resp['ERROR_CODE'],
+                                   'Server responded with status code :{0} and message:'.format(response.status_code,
+                                                                                                resp['ERROR_MESSAGE']))
             else:
                 raise UnknownError(resp.get('ERROR_CODE'), resp.get('ERROR_MESSAGE'))
         else:
@@ -180,7 +206,8 @@ class MammothConnector(object):
     def add_user_to_account(self, email, full_name, user_permission=USER_PERMISSIONS.ANALYST,
                             get_access_token=False):
         log.info("Account id is {0}".format(self.__account_id))
-        response = self._make_signed_request('post', '/accounts/{account_id}/users'.format(account_id=self.__account_id),
+        response = self._make_signed_request('post',
+                                             '/accounts/{account_id}/users'.format(account_id=self.__account_id),
                                              json={
                                                  'email': email,
                                                  'full_name': full_name,
@@ -206,7 +233,7 @@ class MammothConnector(object):
 
     @handleError
     def refresh_dataset(self, ds_id):
-        response = self._make_signed_request('patch', '/dataset/{0}'.format(ds_id), json={
+        response = self._make_signed_request('patch', '/datasets/{0}'.format(ds_id), json={
             "patch": [{"op": "replace", "path": "data", "value": "refresh"}]})
         return response.json()
 
@@ -217,7 +244,9 @@ class MammothConnector(object):
 
     @handleError
     def delete_identity(self, identity_key, integration_key):
-        response = self._make_signed_request('delete', '/integrations/{0}/identities/{1}'.format(integration_key, identity_key))
+        integration_key = encode_integration_key(integration_key)
+        response = self._make_signed_request('delete',
+                                             '/integrations/{0}/identities/{1}'.format(integration_key, identity_key))
         return response.json()
 
     @handleError
@@ -242,14 +271,37 @@ class MammothConnector(object):
         return response.json()['id']
 
     @handleError
-    def create_webhook_dataset(self, name, is_secure=False):
+    def create_dataset_from_url(self, file_url):
         """
-        Create a webhook dataset in Mammoth Analytics.
-        :param name: Name of the dataset
-        :return: Datasource id
+        Create a dataset from file url
+        :params: file url
+        :return: dataset id
         """
 
-        webhook_ds_param = {"name": name, "origins": "*", "is_secure": is_secure}
+        file_url_ds_param = {"url": file_url}
+        r = self._make_signed_request('post', '/weburls', json=file_url_ds_param)
+        response = r.json()
+
+        if response.get('STATUS') == 'SUCCESS':
+            file_id = response.get('id')
+            self.wait_till_file_processing_get_ds(file_id)
+            ds_id = self.get_ds_for_file(file_id)['id']
+            return ds_id
+        else:
+            raise UnknownError(response.get('ERROR_MESSAGE'))
+
+    @handleError
+    def create_webhook_dataset(self, name, is_secure=False, ds_mode="Replace"):
+        """
+        Create a webhook dataset in Mammoth Analytics.
+        Args:
+            name: Name of the dataset
+            is_secure: Is secure True/False
+            ds_mode: Dataset mode of the dataset Replace/Combine
+        Return:
+            response.json(): Datasource id
+        """
+        webhook_ds_param = {"name": name, "origins": "*", "is_secure": is_secure, "mode": ds_mode}
         response = self._make_signed_request(
             'post', '/webhooks',
             json=webhook_ds_param)
@@ -280,7 +332,9 @@ class MammothConnector(object):
     @handleError
     def list_batches(self, ds_id):
         response = self._make_signed_request('get', '/datasets/{0}/batches'.format(ds_id))
-        return response.json()['batches']
+        future_id = response.json()['future_id']
+        future_response = self.track_future_id(future_id)
+        return future_response
 
     @handleError
     def delete_batches(self, ds_id, batches):
@@ -289,14 +343,16 @@ class MammothConnector(object):
         return response.json()
 
     @handleError
-    def get_dataset(self, ds_id):
+    def get_dataset(self, ds_id, include_row_count=True):
         """
         Returns a dataset information dictionary for the given ID
         :param ds_id: The datasource id
         :return: dictionary containing information on the dataset
         """
-        response = self._make_signed_request('get', '/datasets/{0}'.format(ds_id))
-        return response.json()
+        response = self._make_signed_request('get', '/datasets/{0}?INCLUDE_ROW_COUNT={1}'.format(ds_id, include_row_count))
+        future_id = response.json()['future_id']
+        future_response = self.track_future_id(future_id)
+        return future_response
 
     @handleError
     def get_batch(self, ds_id, batch_id, columns=None, condition=None, limit=None, offset=None):
@@ -309,15 +365,21 @@ class MammothConnector(object):
             'limit': limit,
             'offset': offset
         }
-        response = self._make_signed_request('get', '/ datasets / {dataset_id} / batches / {batch_id}'.format(dataset_id=ds_id, batch_id=batch_id),
+        response = self._make_signed_request('get',
+                                             '/ datasets / {dataset_id} / batches / {batch_id}'.format(dataset_id=ds_id,
+                                                                                                       batch_id=batch_id),
                                              json=paging_params)
 
-        return response.json()
+        future_id = response.json()['future_id']
+        future_response = self.track_future_id(future_id)
+        return future_response
 
     @handleError
     def get_task(self, view_id):
         response = self._make_signed_request('get', '/workspaces/{0}/tasks'.format(view_id))
-        return response.json().get('tasks')
+        future_id = response.json()['future_id']
+        future_response = self.track_future_id(future_id)
+        return future_response.get('tasks')
 
     @handleError
     def add_task(self, view_id, params, wait_till_ready=True):
@@ -330,7 +392,7 @@ class MammothConnector(object):
     @handleError
     def update_task(self, view_id, task_id, params, wait_till_ready=True):
         r = self._make_signed_request('patch', '/workspaces/{0}/tasks/{1}'.format(view_id, task_id),
-                                             json={"patch": [{"op": "replace", "path": "params", "value": params}]})
+                                      json={"patch": [{"op": "replace", "path": "params", "value": params}]})
         response = r.json()
         if wait_till_ready:
             response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
@@ -412,8 +474,16 @@ class MammothConnector(object):
     def append_datasets(self, source_dataset_id, target_dataset_id, column_mapping, change_map=None,
                         new_ds_params=None):
         """
-        Method to combine two datasets
+        Method to combine two datasets While appending datasets, target ds gets combined and new rows gets added.At
+        this point, ds updated_at will change. Thereafter, the pipeline re-run happens in all workspaces and
+        updated_at gets modified for all the workspace once after pipeline re-run. Therefore, we need wait till
+        all workspaces updated_at gets changed. -
+        1. Loop until updated_at not greater than last_updated_at. Once updated_at is greater than
+        last_updated_at break the loop and return the response
+        2. Try until retry_count is less than 50, to avoid stuck issue in worse case
         """
+        wksp_data = []
+        date_format = '%Y-%m-%dT%H:%M:%SZ'
         params = {
             "source": "datasource",
             "source_id": source_dataset_id,
@@ -423,9 +493,50 @@ class MammothConnector(object):
             params.update({"change_map": change_map})
         if new_ds_params:
             params.update({"new_ds_params": new_ds_params})
+
+        # List all workspaces of target dataset
+        wksps = self.list_views(target_dataset_id)
+        for wksp in wksps:
+            view_id = int(wksp["id"])
+            res = self.get_view_details(view_id)
+            last_updated_at = res['updated_at']
+            # store all workspace id and last_data_updated_at time into a dictionary
+            wksp_data.append({'view_id': view_id, 'last_updated_at': last_updated_at})
+
+        # append dataset
         response = self._make_signed_request('post', '/datasets/{0}/batches'.format(target_dataset_id), json=params)
-        log.info("The response of append datasets is  {} ".format(response.json()))
-        return response.json()
+        if response.json()['STATUS'] == 'SUCCESS':
+            self.sync_now(target_dataset_id)
+            time.sleep(5)
+            for view_details in wksp_data:
+                view_id = view_details['view_id']
+                last_updated_at = view_details['last_updated_at']
+                res = self.get_view_details(view_id)
+                updated_at = res['updated_at']
+
+                # convert string to datetime
+                last_updated_at = datetime.strptime(last_updated_at, date_format)
+                updated_at = datetime.strptime(updated_at, date_format)
+                retry_count = 0
+
+                while not (updated_at > last_updated_at) and retry_count < MAX_RETRIES:
+                    # iterate until data_updated_at is not greater than last_data_updated_at
+                    retry_count += 1
+                    time.sleep(5)
+                    res = self.get_view_details(view_id)
+                    updated_at = res['updated_at']
+                    updated_at = datetime.strptime(updated_at, date_format)
+
+                if retry_count == MAX_RETRIES:
+                    raise RuntimeError(f"Max retry limit reached to combine datasets.")
+
+            self.sync_now(target_dataset_id)
+            time.sleep(15)
+            r = self._make_signed_request('get', '/datasets/{0}/batches'.format(target_dataset_id))
+            result = r.json()['STATUS']
+            time.sleep(2)
+            if result == 'SUCCESS':
+                return response.json()
 
     def wait_till_file_processing_get_ds(self, file_id):
         """
@@ -435,7 +546,7 @@ class MammothConnector(object):
         :return: A dataset dictionary
         """
         retry_count = 0
-        while retry_count < MAX_RETRIES*4:
+        while retry_count < MAX_RETRIES * 4:
             response = self._make_signed_request('get', '/files/{0}'.format(file_id))
             file_info = response.json()['file']
             status = file_info['status']
@@ -445,7 +556,7 @@ class MammothConnector(object):
             else:
                 time.sleep(RETRY_DELAY_IN_SEC)
             retry_count += 1
-        if retry_count == MAX_RETRIES*4:
+        if retry_count == MAX_RETRIES * 4:
             raise RuntimeError(f"Max time limit reached to get processed file .")
         response = self._make_signed_request('get', '/files/{0}'.format(file_id))
         file_info = response.json()['file']
@@ -466,7 +577,7 @@ class MammothConnector(object):
     @handleError
     def _wait_for_ds_status(self, ds_id, status, check_for_status_equality=True):
         retry_count = 0
-        while ds_id and retry_count < MAX_RETRIES*4:
+        while ds_id and retry_count < MAX_RETRIES * 4:
             ds = self.get_dataset(ds_id)
             ds_status = ds['status']
             if check_for_status_equality:
@@ -516,15 +627,29 @@ class MammothConnector(object):
     @handleError
     def create_view(self, ds_id, duplicate_from_view_id=None):
         """
-        Returns a list of views a dataset has
+        Returns a workspace_id of the dataset
         :param ds_id: Dataset ID
-        :return: list of workspace dictionaries
+        :param duplicate_from_view_id: duplicate existing view
+        :return: workspace_id
         """
-        params = {}
-        if duplicate_from_view_id:
-            params['clone_config_from'] = duplicate_from_view_id
-        response = self._make_signed_request('post', '/datasets/{0}/workspaces'.format(ds_id), json=params)
-        return response.json()['workspace_id']
+        try:
+            params = {}
+            if duplicate_from_view_id:
+                params = {'clone_config_from': duplicate_from_view_id}
+            response = self._make_signed_request('post', '/datasets/{0}/workspaces'.format(ds_id), json=params)
+
+            # View duplication takes a while to reflect the data
+            if 'workspace_id' not in response.json():
+                time.sleep(10)
+                views = self.list_views(ds_id)
+                view_ids = [view['id'] for view in views]
+                workspace_id = max(view_ids)
+                self.wait_for_view_to_finish_processing(workspace_id)
+            else:
+                workspace_id = response.json()['workspace_id']
+        except Exception as e:
+            raise e
+        return workspace_id
 
     @handleError
     def delete_view(self, view_id, wait_till_ready=True):
@@ -540,6 +665,19 @@ class MammothConnector(object):
             response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
         return response
 
+    @handleError
+    def reset_view(self, view_id, wait_till_ready=True):
+        """
+        Make API call to reset the view - Resetting the View will remove all Transformations, Elements and Filters
+        :param view_id: workspace_id
+        :return: response
+        """
+        r = self._make_signed_request('post', '/workspaces/{0}/reset'.format(view_id), json={})
+        response = r.json()
+        if wait_till_ready:
+            response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
+        return response
+
     @handleError
     def rerun_pipeline(self, view_id, force_run_pipeline=False):
         params = {'force_run': force_run_pipeline}
@@ -568,13 +706,13 @@ class MammothConnector(object):
             'offset': offset
         }
 
-        response = self._make_signed_request('get', 'workspaces/{workspace_id}/data'.format(workspace_id=view_id), json=paging_params)
+        response = self._make_signed_request('get', 'workspaces/{workspace_id}/data'.format(workspace_id=view_id),
+                                             json=paging_params)
         future_id = response.json()['future_id']
         future_response = self.track_future_id(future_id)
         log.debug("The future response is {}".format(future_response))
         return future_response['data']
 
-
     @handleError
     def run_transform(self, view_id, param):
         r = self._make_signed_request('post', '/workspaces/{0}/tasks'.format(view_id), json={'param': param})
@@ -583,12 +721,16 @@ class MammothConnector(object):
     @handleError
     def get_view_details(self, view_id):
         r = self._make_signed_request('get', '/workspaces/{0}'.format(view_id))
-        return r.json()
+        future_id = r.json()['future_id']
+        future_response = self.track_future_id(future_id)
+        return future_response
 
     @handleError
     def set_view_properties(self, view_id, properties):
         r = self._make_signed_request('post', '/workspaces/{0}'.format(view_id), json=properties)
-        return r.json()
+        future_id = r.json()['future_id']
+        future_response = self.track_future_id(future_id)
+        return future_response
 
     @handleError
     def wait_for_view_to_finish_processing(self, view_id):
@@ -602,17 +744,18 @@ class MammothConnector(object):
             retry_count += 1
         if retry_count == MAX_RETRIES * 2:
             raise RuntimeError(f"Max retry limit reached to get view in ready state")
+
     @handleError
     def wait_till_view_is_ready(self, view_id):
         retry_count = 0
-        while retry_count < MAX_RETRIES*2:
+        while retry_count < MAX_RETRIES * 2:
             if retry_count != 0:
                 time.sleep(RETRY_DELAY_IN_SEC)
             view = self.get_view_details(view_id)
             if view['status'] == 'ready':
                 break
             retry_count += 1
-        if retry_count == MAX_RETRIES*2:
+        if retry_count == MAX_RETRIES * 2:
             raise RuntimeError(f"Max retry limit reached to get view in ready state")
 
     @handleError
@@ -630,7 +773,7 @@ class MammothConnector(object):
                                   json={'clone_config_from': view_id})
 
     @handleError
-    def add_action(self, view_id, param, wait_till_ready=False ):
+    def add_action(self, view_id, param, wait_till_ready=False):
         """
         Adds an action.
         :param view_id: The view on which the action is to be performed
@@ -640,7 +783,7 @@ class MammothConnector(object):
         r = self._make_signed_request('post', '/workspaces/{0}/actions'.format(view_id), json={'param': param})
         response = r.json()['request_id']
         if wait_till_ready:
-           response = self._wait_till_request_is_complete_get_data(view_id, response)
+            response = self._wait_till_request_is_complete_get_data(view_id, response)
         return response
 
     @handleError
@@ -652,7 +795,8 @@ class MammothConnector(object):
         :param param: The param for the action.
         :return: Request Id for the action
         """
-        r = self._make_signed_request('post', '/workspaces/{0}/actions/{1}'.format(view_id,action_id), json={'param': param})
+        r = self._make_signed_request('post', '/workspaces/{0}/actions/{1}'.format(view_id, action_id),
+                                      json={'param': param})
         response = r.json()['request_id']
         if wait_till_ready:
             response = self._wait_till_request_is_complete_get_data(view_id, r.json()['request_id'])
@@ -669,10 +813,11 @@ class MammothConnector(object):
         :return: Request Id for the action
         """
 
-        r = self._make_signed_request('patch', '/workspaces/{0}/actions/{1}'.format(view_id, action_id), json={"patch": [{"op": "replace", "path": "params", "value": param}]})
+        r = self._make_signed_request('patch', '/workspaces/{0}/actions/{1}'.format(view_id, action_id),
+                                      json={"patch": [{"op": "replace", "path": "params", "value": param}]})
         response = r.json()
         if wait_till_ready:
-           response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
+            response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
         return response['request_id']
 
     def _wait_till_request_is_complete_get_data(self, view_id, request_id):
@@ -680,7 +825,7 @@ class MammothConnector(object):
         while retry_count < MAX_RETRIES:
             if retry_count != 0:
                 time.sleep(RETRY_DELAY_IN_SEC)
-            r = self._make_signed_request('get','/workspaces/{0}/requests/{1}'.format(view_id, request_id))
+            r = self._make_signed_request('get', '/workspaces/{0}/requests/{1}'.format(view_id, request_id))
             data = r.json()
             log.debug("data in _wait_till_request_is_complete_get_data {}".format(data))
             if data['status'] in ['success', 'failure', 'referror', 'error']:
@@ -732,7 +877,7 @@ class MammothConnector(object):
     @handleError
     def download_view_as_csv(self, view_id, file_prefix=None, condition=None):
         if not isinstance(file_prefix, str):
-            file_prefix ='file_{0}'.format(random.randint(1000, 2000))
+            file_prefix = 'file_{0}'.format(random.randint(1000, 2000))
 
         action_param = {
             "handler_type": "s3",
@@ -745,7 +890,7 @@ class MammothConnector(object):
                 "file": file_prefix,
                 "use_format": True,
                 "include_hidden": False
-             }
+            }
         }
         action_response = self.add_action(view_id, action_param, wait_till_ready=True)
         log.debug("The response of download_view_as_csv is {}".format(action_response))
@@ -770,6 +915,7 @@ class MammothConnector(object):
 
     @handleError
     def add_third_party_identity(self, integration_key, identity_config):
+        integration_key = encode_integration_key(integration_key)
         url = '/integrations/{0}/identities'.format(integration_key)
         r = self._make_signed_request('post', url, json=identity_config)
         request_id = r.json()['request_id']
@@ -791,6 +937,7 @@ class MammothConnector(object):
 
     @handleError
     def get_third_party_identity_key_by_name(self, integration_key, name):
+        integration_key = encode_integration_key(integration_key)
         url = "/integrations/{0}/identities".format(integration_key)
         r = self._make_signed_request('get', url)
         data = r.json()
@@ -800,6 +947,7 @@ class MammothConnector(object):
 
     @handleError
     def get_third_party_identities(self, integration_key):
+        integration_key = encode_integration_key(integration_key)
         url = "/integrations/{0}/identities".format(integration_key)
         r = self._make_signed_request('get', url)
         data = r.json()
@@ -807,6 +955,7 @@ class MammothConnector(object):
 
     @handleError
     def validate_third_party_ds_config(self, integration_key, identity_key, ds_config):
+        integration_key = encode_integration_key(integration_key)
         url = '/integrations/{0}/identities/{1}/dsConfigs'.format(integration_key, identity_key)
         r = self._make_signed_request('post', url, json=ds_config)
         data = r.json()
@@ -825,7 +974,7 @@ class MammothConnector(object):
 
     def wait_till_ds_ready(self, ds_id):
         retry_count = 0
-        while retry_count < MAX_RETRIES*4:
+        while retry_count < MAX_RETRIES * 4:
             datasets = self.list_datasets()
             ds_ready = False
             for i in range(len(datasets)):
@@ -843,7 +992,7 @@ class MammothConnector(object):
             if ds_ready:
                 break
             retry_count += 1
-        if retry_count == MAX_RETRIES*4:
+        if retry_count == MAX_RETRIES * 4:
             raise RuntimeError(f"Max time limit reached to get ds ready.")
 
     @handleError
@@ -872,7 +1021,7 @@ class MammothConnector(object):
             if retry_count != 0:
                 time.sleep(RETRY_DELAY_IN_SEC)
             resource_resp = self._make_signed_request('get', f'/future/{future_id}')
-            resource_resp_json = resource_resp.json()  
+            resource_resp_json = resource_resp.json()
             future_object = resource_resp_json['future']
             if future_object['status'] != 'processing':
                 response = future_object['response']
@@ -925,7 +1074,8 @@ class MammothConnector(object):
         :param param - List of batch column/s to be added to the dataset
         """
         response = self._make_signed_request('patch', f'/datasets/{ds_id}',
-                                                json={"patch": [{"op": "replace", "path": "batch_columns","value": {"add_columns": param,"remove_columns": []}}]})
+                                             json={"patch": [{"op": "replace", "path": "batch_columns",
+                                                              "value": {"add_columns": param, "remove_columns": []}}]})
         return response.json()
 
     @handleError
@@ -935,14 +1085,14 @@ class MammothConnector(object):
         :param ds_id:
         :return - List of batch column/s keys
         """
-        list_batch_columns = []
+        batch_column_keys = []
         metadata = self.get_dataset(ds_id)['meta_data']
         for value in metadata:
             if value['internal_name'] in RESERVED_BATCH_COLUMN_INTERNAL_NAMES_AND_KEYS.keys():
                 batch_col_key = RESERVED_BATCH_COLUMN_INTERNAL_NAMES_AND_KEYS[value['internal_name']]
-                batch_columns_list.append(batch_col_key)
+                batch_column_keys.append(batch_col_key)
 
-        return list_batch_columns
+        return batch_column_keys
 
     @handleError
     def remove_batch_columns(self, ds_id, rem_cols):
@@ -952,8 +1102,8 @@ class MammothConnector(object):
         :param rem_cols - List of batch column/s (keys) to be removed from the dataset
         """
         # Get batch columns of the dataset
-        batch_columns_list = self.get_batch_column_keys(ds_id)
-        add_cols = list(set(batch_columns_list) - set(rem_cols))
+        batch_column_keys = self.get_batch_column_keys(ds_id)
+        add_cols = list(set(batch_column_keys) - set(rem_cols))
 
         response = self._make_signed_request('patch', '/datasets/{0}'.format(ds_id),
                                              json={"patch": [{"op": "replace", "path": "batch_columns",
@@ -961,5 +1111,120 @@ class MammothConnector(object):
                                                                         "remove_columns": rem_cols}}]})
         return response.json()
 
+    @handleError
+    def add_publish(self, view_id, params, wait_till_ready=True):
+        r = self._make_signed_request('post', '/workspaces/{0}/publish'.format(view_id), json={'param': params})
 
+        response = r.json()
+        if wait_till_ready:
+            response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
+        return response
 
+    @handleError
+    def delete_publish(self, view_id, publish_trigger_id):
+        response = self._make_signed_request('delete', f'/workspaces/{view_id}/publish/{publish_trigger_id}')
+
+        future_id = response.json()['future_id']
+        future_response = self.track_future_id(future_id)
+        return future_response
+
+    @handleError
+    def regenerate_publish_password(self, view_id, publish_odbc_type):
+        param = {
+            "patch": [{
+                "op": "replace", "path": "reset_password",
+                "value": {
+                    "odbc_type": publish_odbc_type
+                }
+            }]
+        }
+        response = self._make_signed_request('patch', \
+                                             f'/workspaces/{view_id}/publish', \
+                                             json=param)
+
+        future_id = response.json()['future_id']
+        future_response = self.track_future_id(future_id)
+        return future_response['password']
+
+    @handleError
+    def get_publish_credentials(self, odbc_type):
+        response = self._make_signed_request('get', f'/publish_credentials?odbc_type={odbc_type}')
+        return response.json()
+
+    @handleError
+    def create_folder(self, params):
+        """
+        Method to create folder
+        :param params - params for the create folder
+        """
+        r = self._make_signed_request('post', '/labels', json=params)
+        response = r.json()
+        return response
+
+    @handleError
+    def move_file_to_folder(self, destination_folder_id, params, wait_till_ready=True):
+        """
+        Method to move files from one folder to another
+        :param destination_folder_id - id of the folder to which files are moving
+        :param params - params which contains files to be moved
+        """
+        response = self._make_signed_request('post', '/labels/{0}/resources'.format(destination_folder_id), json=params)
+        future_id = response.json()['future_id']
+        if wait_till_ready:
+            future_response = self.track_future_id(future_id)
+        return future_response
+
+    @handleError
+    def list_folders(self):
+        """
+        Method to list all the folders
+        """
+        response = self._make_signed_request('get', '/labels')
+        return response.json().get('labels')
+
+    @handleError
+    def delete_folder(self, folder_id):
+        """
+        Method to delete folder
+        :param folder_id - id of the folder to be deleted
+        """
+        r = self._make_signed_request('delete', '/labels/{0}'.format(folder_id))
+        response = r.json()
+        return response
+
+    @handleError
+    def list_resources(self):
+        """
+        Method to list all the resources
+        """
+        response = self._make_signed_request('get', '/resources')
+        data = response.json()
+        return data['resources']
+
+    @handleError
+    def rename_dataset(self, ds_id, dataset_name):
+        """
+        method to rename dataset
+        :param
+            ds_id: dataset id
+            dataset_name: dataset name to be changed
+        """
+        res = self._make_signed_request('patch', '/datasets/{0}'.format(ds_id),
+                                        json={"patch": [{"op": "replace", "path": "name",
+                                                         "value": dataset_name}]})
+        response = res.json()
+        return response
+
+    @handleError
+    def rename_view(self, wksp_id, wksp_name):
+        """
+        method to rename view
+        :param
+            wksp_id: workspace id
+            wksp_name: dataset name to be changed
+        """
+        res = self._make_signed_request('patch', '/workspaces/{0}'.format(wksp_id),
+                                           json={"patch": [{"op": "replace", "path": "name",
+                                                            "value": wksp_name}]})
+        response = res.json()
+        return response
